@@ -1,18 +1,21 @@
 package Ovirt::VM;
+
+use v5.16;
 use LWP::UserAgent;
 use XML::LibXML;
 use Carp;
 use Moo;
 
 with 'Ovirt';
+our $VERSION = '0.02';
 
 =head1 NAME
 
-Ovirt::VM - Bindings with Ovirt VM API 
+Ovirt::VM - Bindings for Ovirt VM API 
 
 =head1 VERSION
 
-Version 0.01
+Version 0.02
 
 =cut
 
@@ -43,10 +46,11 @@ Version 0.01
  $vm->create('vm1','Default','CentOS7');
  $vm->remove('2d83bb51-9a77-432d-939c-35be207017b9');
  
- # start, stop, reboot vm
- $vm->start ('b4738b0f-b73d-4a66-baa8-2ba465d63132');
- $vm->stop  ('b4738b0f-b73d-4a66-baa8-2ba465d63132');
- $vm->reboot('b4738b0f-b73d-4a66-baa8-2ba465d63132');
+ # start, stop, reboot, migrate vm
+ $vm->start     ('b4738b0f-b73d-4a66-baa8-2ba465d63132');
+ $vm->stop      ('b4738b0f-b73d-4a66-baa8-2ba465d63132');
+ $vm->reboot    ('b4738b0f-b73d-4a66-baa8-2ba465d63132');
+ $vm->migrate   ('b4738b0f-b73d-4a66-baa8-2ba465d63132');
 
  # Output also available in hash
  # for example to print all vm name and state
@@ -88,6 +92,7 @@ Version 0.01
                             cluster_id  template_id
                             stop_time   creation_time
                             timezone    usb_enabled
+                            host_id
 =cut
 
 has 'vm_url'                => ( is => 'ro', default => '/api/vms' );
@@ -112,6 +117,7 @@ has 'vm_output_attrs'       => ( is => 'rw', default => 'id,name,state',
                                                                 cluster_id  template_id
                                                                 stop_time   creation_time
                                                                 timezone    usb_enabled
+                                                                host_id
                                                             |;
                                      for my $attr (@attrs) {
                                          $attr = lc ($attr);
@@ -184,6 +190,9 @@ sub vm_action_output {
     
     $self->log->debug($xml);
     
+    return $xml;
+    
+=comment
     my $status    = $xml->findnodes('/action/status/state');
 
     if ($status eq 'complete') {
@@ -200,6 +209,7 @@ sub vm_action_output {
         print "reason   : $reason\n";
         print "detail   : $detail\n";
     }
+=cut
 }
 
 =head2 remove
@@ -330,6 +340,9 @@ EOF
     
     $self->log->debug($output);
     
+    return $output;
+    
+=comment
     if ($action->is_success) {
         my $status    = $output->findnodes('/vm/creation_status/state');
         print "status   : $status\n";
@@ -342,6 +355,7 @@ EOF
         print "status   : $status\n";
         print "detail   : $detail\n";
     }
+=cut
 }
 
 =head2 start
@@ -487,6 +501,54 @@ sub reboot {
     $self->vm_action_output($output);
 }
 
+=head2 migrate
+
+ migrate vm
+ required arguments ($vmid)
+ if $self->id is set during initialization, argument is not required
+=cut
+
+sub migrate {
+    my $self = shift;
+    
+    my $vmid = shift || undef;
+    my $migrate_url;
+    
+    # set the migrate final url
+    if ($self->id) {
+        $migrate_url = $self->url . "/migrate";
+    }
+    else {
+        if ($vmid) {
+            my $is_valid = $self->is_vmid_valid($vmid);
+            croak "vm id not found" unless $is_valid;
+            
+            $migrate_url = $self->url . "/$vmid/migrate";
+        }
+        else {
+            croak "vm id is required";
+        }
+    }
+    
+    $self->log->debug("migrate action url = $migrate_url");
+    
+    # set user agent
+    my $ua      = LWP::UserAgent->new();
+    my $action;
+    
+    if ($self->vm_boot_dev eq 'hd') {
+        $action = $ua->post($migrate_url, Content_Type => 'application/xml', Content => $self->vm_hd_xml);
+    }
+    else {
+        $action = $ua->post($migrate_url, Content_Type => 'application/xml', Content => $self->vm_cdrom_xml);   
+    }
+    
+    my $parser = XML::LibXML->new();
+    my $output = $parser->parse_string($action->decoded_content);
+    
+    $self->vm_action_output($output);
+}
+
 =head2 is_vmid_valid
 
     return false if vmid not valid
@@ -572,7 +634,7 @@ sub list {
             $vmid_element = $element_id;
         }
         
-        croak "vm id not found" unless $vmid_element;
+        croak "vm id not found" unless $vmid_element >= 0;
         
         for my $attr (@attrs) { 
            $self->log->debug("requesting attribute $attr for element $vmid_element");
@@ -700,6 +762,9 @@ sub get_vm_by_element_id {
     elsif   ($attr eq 'usb_enabled') {
             return $self->hash_output->{vm}[$element_id]->{usb}->{enabled};
     }
+    elsif   ($attr eq 'host_id') {
+            return $self->hash_output->{vm}[$element_id]->{host}->{id};
+    }
 }
 
 =head2 get_vm_by_self_id
@@ -780,11 +845,14 @@ sub get_vm_by_self_id {
     elsif   ($attr eq 'usb_enabled') {
             return $self->hash_output->{usb}->{enabled};
     }
+    elsif   ($attr eq 'host_id') {
+            return $self->hash_output->{host}->{id};
+    }
 }
 
 =head1 AUTHOR
 
- "Heince Kurniawan", C<< <"heince at gmail.com"> >>
+ "Heince Kurniawan", C<< <"heince at cpan.org"> >>
 
 =head1 BUGS
 
